@@ -55,9 +55,9 @@ using namespace Efont;
 #define QUERY_FAMILY_OPT	327
 #define INFO_OPT		328
 
-Clp_Option options[] = {
+const Clp_Option options[] = {
     
-    { "script", 0, SCRIPT_OPT, Clp_ArgString, 0 },
+    { "script", 0, SCRIPT_OPT, Clp_ValString, 0 },
     { "quiet", 'q', QUIET_OPT, 0, Clp_Negate },
     { "verbose", 'V', VERBOSE_OPT, 0, Clp_Negate },
     { "features", 'f', QUERY_FEATURES_OPT, 0, 0 },
@@ -358,6 +358,14 @@ do_info(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *result_errh
 		sa << "Full name:           " << s << "\n";
 	    if (String s = name.english_name(OpenType::Name::N_POSTSCRIPT))
 		sa << "PostScript name:     " << s << "\n";
+	    if (String s = name.english_name(OpenType::Name::N_POSTSCRIPT_CID))
+		sa << "PostScript CID name: " << s << "\n";
+	    if (String s = name.english_name(OpenType::Name::N_PREF_FAMILY))
+		sa << "Preferred family:    " << s << "\n";
+	    if (String s = name.english_name(OpenType::Name::N_PREF_SUBFAMILY))
+		sa << "Preferred subfamily: " << s << "\n";
+	    if (String s = name.english_name(OpenType::Name::N_MAC_COMPAT_FULLNAME))
+		sa << "Mac font menu name:  " << s << "\n";
 	    if (String s = name.english_name(OpenType::Name::N_VERSION))
 		sa << "Version:             " << s << "\n";
 	    if (String s = name.english_name(OpenType::Name::N_UNIQUEID))
@@ -380,11 +388,13 @@ do_info(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *result_errh
 		sa << "License URL:         " << s << "\n";
 	    if (String s = name.english_name(OpenType::Name::N_LICENSE_DESCRIPTION))
 		sa << "License Description: " << s << "\n";
+	    if (String s = name.english_name(OpenType::Name::N_SAMPLE_TEXT))
+		sa << "Sample text:         " << s << "\n";
 	}
     }
 
-    if (errh->nerrors() == before_nerrors)
-	result_errh->message("%s", (sa ? sa.c_str() : "no designer information"));
+    if (errh->nerrors() == before_nerrors) 
+	result_errh->message("%s", (sa ? sa.c_str() : "no name information"));
 }
 
 static void
@@ -454,7 +464,7 @@ main(int argc, char *argv[])
 	  case SCRIPT_OPT: {
 	      if (!script.null())
 		  usage_error(errh, "--script already specified");
-	      String arg = clp->arg;
+	      String arg = clp->vstr;
 	      int period = arg.find_left('.');
 	      OpenType::Tag scr(period <= 0 ? arg : arg.substring(0, period));
 	      if (scr.valid() && period > 0) {
@@ -511,7 +521,7 @@ particular purpose.\n");
 	    break;
 
 	  case Clp_NotOption:
-	    input_files.push_back(clp->arg);
+	    input_files.push_back(clp->vstr);
 	    break;
 
 	  case Clp_Done:
