@@ -65,6 +65,7 @@ enum { U_EXCLAMDOWN = 0x00A1,	// U+00A1 INVERTED EXCLAMATION MARK
        U_INTERROBANGDOWN = 0xD80B, // invalid Unicode
        U_TWELVEUDASH = 0xD80C,	// invalid Unicode
        U_RINGFITTED = 0xD80D,	// invalid Unicode
+       U_USE_KERNX = 0xD80E,	// invalid Unicode, not in maps
        U_VS1 = 0xFE00,
        U_VS16 = 0xFE0F,
        U_VS17 = 0xE0100,
@@ -218,8 +219,8 @@ bool
 Secondary::encode_uni(int code, PermString name, uint32_t uni, Metrics &metrics, ErrorHandler *errh)
 {
     Vector<Setting> v;
-    if (setting(uni, v, metrics, errh)) {
-	metrics.encode_virtual(code, name, uni, v);
+    if (int s = setting(uni, v, metrics, errh)) {
+	metrics.encode_virtual(code, name, uni, v, s > 1);
 	return true;
     } else if (_next)
 	return _next->encode_uni(code, name, uni, metrics, errh);
@@ -237,7 +238,7 @@ T1Secondary::T1Secondary(const FontInfo &finfo, const String &font_name,
 	_spacewidth = width;
 }
 
-bool
+int
 Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHandler *errh)
 {
     if (_next)
@@ -254,6 +255,13 @@ T1Secondary::char_setting(Vector<Setting> &v, Metrics &metrics, int uni, ...)
     // collect codes
     va_list val;
     va_start(val, uni);
+
+    int kerntype = Setting::KERN;
+    if (uni == U_USE_KERNX) {
+	kerntype = Setting::KERNX;
+	uni = va_arg(val, int);
+    }
+    
     for (; uni; uni = va_arg(val, int)) {
 	int code = metrics.unicode_encoding(uni);
 	if (code < 0) {
@@ -268,7 +276,7 @@ T1Secondary::char_setting(Vector<Setting> &v, Metrics &metrics, int uni, ...)
     // generate setting
     for (int i = 0; i < codes.size(); i++) {
 	if (i)
-	    v.push_back(Setting(Setting::KERN));
+	    v.push_back(Setting(kerntype));
 	v.push_back(Setting(Setting::SHOW, codes[i], metrics.base_glyph(codes[i])));
     }
     return true;
@@ -281,7 +289,7 @@ T1Secondary::encode_uni(int code, PermString name, uint32_t uni, Metrics &metric
 	Vector<Setting> v;
 	setting(uni, v, metrics, errh);
 	int which = (uni == U_ALTSELECTOR ? 0 : (uni <= U_VS16 ? uni - U_VS1 + 1 : uni - U_VS17 + 17));
-	metrics.encode_virtual(code, (which ? permprintf("<vs%d>", which) : PermString("<altselector>")), uni, v);
+	metrics.encode_virtual(code, (which ? permprintf("<vs%d>", which) : PermString("<altselector>")), uni, v, false);
 	metrics.add_altselector_code(code, which);
 	return true;
     } else
@@ -302,13 +310,23 @@ T1Secondary::dotlessj_font(Metrics &metrics, ErrorHandler *errh, Glyph &dj_glyph
 {
     if (!_font_name || !_finfo.otf || !_finfo.cff)
 	return -1;
-    
-    String dj_name = suffix_font_name(_font_name, "--lcdfj");
+
+    String dj_name;
+    bool install_metrics;
+    // XXX make sure dotlessj is for the main font?
+    if ((dj_name = installed_metrics_font_name(_font_name, "dotlessj")))
+	install_metrics = false;
+    else {
+	dj_name = suffix_font_name(_font_name, "--lcdfj");
+	install_metrics = true;
+    }
+
+    // is dotlessj already mapped?
     for (int i = 0; i < metrics.n_mapped_fonts(); i++)
 	if (metrics.mapped_font_name(i) == dj_name)
 	    return i;
     
-    if (String filename = installed_type1_dotlessj(_otf_file_name, _finfo.cff->font_name(), (output_flags & G_DOTLESSJ) && (output_flags & G_TYPE1), errh)) {
+    if (String filename = installed_type1_dotlessj(_otf_file_name, _finfo.cff->font_name(), (output_flags & G_DOTLESSJ), errh)) {
 
 	// check for special case: "\0" means the font's "j" is already
 	// dotless
@@ -339,21 +357,25 @@ T1Secondary::dotlessj_font(Metrics &metrics, ErrorHandler *errh, Glyph &dj_glyph
 	    return -1;
 	}
 
-	// create metrics for dotless-J
-	Metrics dj_metrics(font, 256);
+	// find dotless-J character
 	Vector<PermString> glyph_names;
 	font->glyph_names(glyph_names);
 	Vector<PermString>::iterator g = std::find(glyph_names.begin(), glyph_names.end(), "uni0237");
-	if (g != glyph_names.end()) {
-	    dj_glyph = g - glyph_names.begin();
-	    dj_metrics.encode('j', U_DOTLESSJ, dj_glyph);
-	} else {
+	if (g == glyph_names.end()) {
 	    errh->error("%s: dotless-J font has no 'uni0237' glyph", filename.c_str());
 	    delete font;
 	    return -1;
 	}
-	::dotlessj_file_name = filename;
-	output_metrics(dj_metrics, font->font_name(), -1, _finfo, String(), String(), dj_name, dotlessj_dvips_include, errh);
+	dj_glyph = g - glyph_names.begin();
+	
+	// create metrics for dotless-J
+	if (install_metrics) {
+	    Metrics dj_metrics(font, 256);
+	    dj_metrics.encode('j', U_DOTLESSJ, dj_glyph);
+	    ::dotlessj_file_name = filename;
+	    output_metrics(dj_metrics, font->font_name(), -1, _finfo, String(), String(), dj_name, dotlessj_dvips_include, errh);
+	} else if (verbose)
+	    errh->message("using '%s' for dotless-J font metrics", dj_name.c_str());
 	
 	// add font to metrics
 	return metrics.add_mapped_font(font, dj_name);
@@ -362,26 +384,27 @@ T1Secondary::dotlessj_font(Metrics &metrics, ErrorHandler *errh, Glyph &dj_glyph
 	return -1;
 }
 
-bool
+int
 T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHandler *errh)
 {
     Transform xform;
     int vsize = v.size();
+    extern int letterspace;
     
     switch (uni) {
 	
       case U_CWM:
       case U_ALTSELECTOR:
 	v.push_back(Setting(Setting::RULE, 0, _xheight));
-	return true;
+	return 1;
 
       case U_CAPITALCWM:
 	v.push_back(Setting(Setting::RULE, 0, font_cap_height(_finfo, xform)));
-	return true;
+	return 1;
 
       case U_ASCENDERCWM:
 	v.push_back(Setting(Setting::RULE, 0, font_ascender(_finfo, xform)));
-	return true;
+	return 1;
 
       case U_VISIBLESPACE:
 	v.push_back(Setting(Setting::MOVE, 50, -150));
@@ -389,63 +412,63 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	v.push_back(Setting(Setting::RULE, _spacewidth, 40));
 	v.push_back(Setting(Setting::RULE, 40, 150));
 	v.push_back(Setting(Setting::MOVE, 50, 150));
-	return true;
+	return 2;
 
       case U_SS:
 	if (char_setting(v, metrics, 'S', 'S', 0))
-	    return true;
+	    return 1;
 	break;
 
       case U_SSSMALL:
 	if (char_setting(v, metrics, U_SSMALL, U_SSMALL, 0))
-	    return true;
+	    return 1;
 	else if (char_setting(v, metrics, 's', 's', 0))
-	    return true;
+	    return 1;
 	break;
 
       case U_FFSMALL:
 	if (char_setting(v, metrics, U_FSMALL, U_FSMALL, 0))
-	    return true;
+	    return 1;
 	else if (char_setting(v, metrics, 'f', 'f', 0))
-	    return true;
+	    return 1;
 	break;
 
       case U_FISMALL:
 	if (char_setting(v, metrics, U_FSMALL, U_ISMALL, 0))
-	    return true;
+	    return 1;
 	else if (char_setting(v, metrics, 'f', 'i', 0))
-	    return true;
+	    return 1;
 	break;
 
       case U_FLSMALL:
 	if (char_setting(v, metrics, U_FSMALL, U_LSMALL, 0))
-	    return true;
+	    return 1;
 	else if (char_setting(v, metrics, 'f', 'l', 0))
-	    return true;
+	    return 1;
 	break;
 
       case U_FFISMALL:
 	if (char_setting(v, metrics, U_FSMALL, U_FSMALL, U_ISMALL, 0))
-	    return true;
+	    return 1;
 	else if (char_setting(v, metrics, 'f', 'f', 'i', 0))
-	    return true;
+	    return 1;
 	break;
 
       case U_FFLSMALL:
 	if (char_setting(v, metrics, U_FSMALL, U_FSMALL, U_LSMALL, 0))
-	    return true;
+	    return 1;
 	else if (char_setting(v, metrics, 'f', 'f', 'l', 0))
-	    return true;
+	    return 1;
 	break;
 
       case U_IJ:
 	if (char_setting(v, metrics, 'I', 'J', 0))
-	    return true;
+	    return 1;
 	break;
 
       case U_ij:
 	if (char_setting(v, metrics, 'i', 'j', 0))
-	    return true;
+	    return 1;
 	break;
 
       case U_DOTLESSJ:
@@ -456,9 +479,9 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	  if (which >= 0) {
 	      v.push_back(Setting(Setting::FONT, which));
 	      v.push_back(Setting(Setting::SHOW, 'j', dj_glyph));
-	      return true;
+	      return 2;
 	  } else if (which == J_NODOT && char_setting(v, metrics, 'j', 0))
-	      return true;
+	      return 1;
 	  break;
       }
 
@@ -466,10 +489,10 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	if (char_setting(v, metrics, '[', 0)) {
 	    if (!_finfo.is_fixed_pitch()) {
 		double d = char_one_bound(_finfo, xform, 4, true, 0, '[', 0);
-		v.push_back(Setting(Setting::MOVE, (int) (-0.666 * d), 0));
+		v.push_back(Setting(Setting::MOVE, (int) (-0.666 * d - letterspace), 0));
 	    }
 	    char_setting(v, metrics, '[', 0);
-	    return true;
+	    return 1;
 	}
 	break;
 
@@ -477,10 +500,10 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	if (char_setting(v, metrics, ']', 0)) {
 	    if (!_finfo.is_fixed_pitch()) {
 		double d = char_one_bound(_finfo, xform, 4, true, 0, ']', 0);
-		v.push_back(Setting(Setting::MOVE, (int) (-0.666 * d), 0));
+		v.push_back(Setting(Setting::MOVE, (int) (-0.666 * d - letterspace), 0));
 	    }
 	    char_setting(v, metrics, ']', 0);
-	    return true;
+	    return 1;
 	}
 	break;
 	
@@ -488,10 +511,10 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	if (char_setting(v, metrics, '|', 0)) {
 	    if (!_finfo.is_fixed_pitch()) {
 		double d = char_one_bound(_finfo, Transform(), 4, true, 0, '|', 0);
-		v.push_back(Setting(Setting::MOVE, (int) (-0.333 * d), 0));
+		v.push_back(Setting(Setting::MOVE, (int) (-0.333 * d - letterspace), 0));
 	    }
 	    char_setting(v, metrics, '|', 0);
-	    return true;
+	    return 1;
 	}
 	break;
 
@@ -503,8 +526,10 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	  if (char_bounds(bounds, bounds[4], _finfo, xform, '('))
 	      dropdown -= std::max(bounds[3], 0) + std::min(bounds[1], 0);
 	  v.push_back(Setting(Setting::MOVE, 0, (int) (-dropdown / 2)));
-	  if (char_setting(v, metrics, '*', 0))
-	      return true;
+	  if (char_setting(v, metrics, '*', 0)) {
+	      v.push_back(Setting(Setting::MOVE, 0, -(int) (-dropdown / 2)));
+	      return 1;
+	  }
 	  break;
       }
 
@@ -512,10 +537,10 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	if (char_setting(v, metrics, U_ENDASH, 0)) {
 	    if (!_finfo.is_fixed_pitch()) {
 		double d = char_one_bound(_finfo, xform, 4, true, 0, U_ENDASH, 0);
-		v.push_back(Setting(Setting::MOVE, (int) (667 - 2 * d), 0));
+		v.push_back(Setting(Setting::MOVE, (int) (667 - 2 * d - letterspace), 0));
 	    }
 	    char_setting(v, metrics, U_ENDASH, 0);
-	    return true;
+	    return 1;
 	}
 	break;
 	
@@ -523,17 +548,17 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	if (char_setting(v, metrics, U_ENDASH, 0)) {
 	    if (!_finfo.is_fixed_pitch()) {
 		double d = char_one_bound(_finfo, xform, 4, true, 0, U_ENDASH, 0);
-		v.push_back(Setting(Setting::MOVE, (int) (750 - 2 * d), 0));
+		v.push_back(Setting(Setting::MOVE, (int) (750 - 2 * d - letterspace), 0));
 	    }
 	    char_setting(v, metrics, U_ENDASH, 0);
-	    return true;
+	    return 1;
 	}
 	break;
 	
       case U_CENTIGRADE:
         // TODO: set italic correction to that of a 'C'
-	if (char_setting(v, metrics, U_DEGREE, 'C', 0))
-	    return true;
+	if (char_setting(v, metrics, U_USE_KERNX, U_DEGREE, 'C', 0))
+	    return 1;
 	break;
 	
       case U_INTERROBANG: {
@@ -545,7 +570,7 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	  if (char_setting(v, metrics, '!', 0)) {
 	      v.push_back(Setting(Setting::POP));
 	      if (char_setting(v, metrics, '?', 0))
-		  return true;
+		  return 1;
 	  }
 	  break;
       }
@@ -559,14 +584,14 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	  if (char_setting(v, metrics, U_EXCLAMDOWN, 0)) {
 	      v.push_back(Setting(Setting::POP));
 	      if (char_setting(v, metrics, U_QUESTIONDOWN, 0))
-		  return true;
+		  return 1;
 	  }
 	  break;
       }
 
       case U_PERTENTHOUSAND:
-	if (char_setting(v, metrics, 0xF661, U_FRACTION, 0xF655, 0xF655, 0xF655, 0))
-	    return true;
+	if (char_setting(v, metrics, U_USE_KERNX, 0xF661, U_FRACTION, 0xF655, 0xF655, 0xF655, 0))
+	    return 1;
 	break;
 
       case U_RINGFITTED: {
@@ -582,7 +607,7 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	      v.push_back(Setting(Setting::MOVE, offset, 0));
 	      if (char_setting(v, metrics, ring_char, 0)) {
 		  v.push_back(Setting(Setting::MOVE, A_width - ring_width - offset, 0));
-		  return true;
+		  return 1;
 	      }
 	  }
 	  break;
